@@ -6,37 +6,32 @@ use strict;
 use 5.010000;
 
 use Exporter qw( import );
-our @EXPORT_OK = qw( get_yt_download_info_as_h_ref get_new_video_url get_download_info_as_json );
+our @EXPORT_OK = qw( wrapper_get get_new_video_url get_download_info_as_json );
 
+use File::Which         qw( which );
 use IPC::System::Simple qw( capture );
-use JSON                qw( decode_json );
 use LWP::UserAgent      qw();
 use Term::ANSIScreen    qw( :screen );
 use Try::Tiny           qw( try catch );
 
 use if $^O eq 'MSWin32', 'Win32::Console::ANSI';
 
-use App::YTDL::Helper qw( HIDE_CURSOR SHOW_CURSOR );
+sub HIDE_CURSOR () { "\e[?25l" }
+sub SHOW_CURSOR () { "\e[?25h" }
 
 
 
-sub get_yt_download_info_as_h_ref {
+sub wrapper_get {
     my ( $opt, $url ) = @_;
     my $show_progress = 1;
-    my $ua = LWP::UserAgent->new(
-        agent         => $opt->{useragent},
-        timeout       => $opt->{timeout},
-        show_progress => $show_progress
-    );
-    my $h_ref;
+    my $ua = LWP::UserAgent->new( agent => $opt->{useragent}, timeout => $opt->{timeout}, show_progress => $show_progress );
+    my $res;
     my $count = 1;
     RETRY: while ( 1 ) {
         my $not_ok;
         try {
-            my $res = $ua->get( $url );
+            $res = $ua->get( $url );
             die $res->status_line, ': ', $url if ! $res->is_success;
-            my $json = $res->decoded_content;
-            $h_ref = decode_json( $json );
         }
         catch {
             if ( $count > $opt->{retries} ) {
@@ -45,17 +40,19 @@ sub get_yt_download_info_as_h_ref {
             say "$count/$opt->{retries}  $_";
             $count++;
             $not_ok = 1;
-            sleep $opt->{retries} * 3;
+            sleep $opt->{retries} * 2;
         };
         next RETRY if $not_ok;
-        return $h_ref;
+        return $res;
     }
 }
 
 
 sub get_new_video_url {
-    my ( $opt, $info, $ex, $video_id, $fmt ) = @_;
-    my @cmd = ( $opt->{youtube_dl} );
+    my ( $opt, $info, $video_id ) = @_;
+    my $fmt = $info->{$video_id}{fmt};
+    my $youtube_dl = which( 'youtube-dl' ) // 'youtube-dl';
+    my @cmd = ( $youtube_dl );
     push @cmd, '--user-agent', $opt->{useragent} if defined $opt->{useragent};
     push @cmd, '--socket-timeout', $opt->{timeout};
     #push @cmd, '-v';
@@ -74,44 +71,41 @@ sub get_new_video_url {
 
 sub get_download_info_as_json {
     my ( $opt, $video_id, $message ) = @_;
-    my @cmd = ( $opt->{youtube_dl} );
+    my $youtube_dl = which( 'youtube-dl' ) // 'youtube-dl';
+    my @cmd = ( $youtube_dl );
     push @cmd, '--user-agent', $opt->{useragent} if defined $opt->{useragent};
     push @cmd, '--socket-timeout', $opt->{timeout};
-    push @cmd, '--youtube-skip-dash-manifest';
     #push @cmd, '-v';
     push @cmd, '--dump-json', '--', $video_id;
-    my $json;
+    my $capture;
     my $count = 1;
-
     RETRY: while ( 1 ) {
         my $not_ok;
         try {
             print HIDE_CURSOR;
             print $message . '...';
-            $json = capture( @cmd );
+            $capture = capture( @cmd );
             print "\r", clline;
             print $message . "done.\n";
             print SHOW_CURSOR;
-            die if ! $json;
+            die if ! defined $capture;
         }
         catch {
             say "$count/$opt->{retries}  $video_id: $_";
             $count++;
             $not_ok = 1;
             print SHOW_CURSOR;
-            sleep $opt->{retries} * 3;
+            sleep $opt->{retries} * 2;
         };
         if ( $count > $opt->{retries} ) {
-            push @{$opt->{error_get_download_infos}}, $video_id; # $ex
+            push @{$opt->{error_get_download_infos}}, $video_id;
             return;
         }
-        if ( $not_ok ) {
-            next RETRY;
-        }
+        next RETRY if $not_ok;
         last;
     }
     $opt->{up}++;
-    return $json;
+    return $capture;
 }
 
 
